#include "homogtrans.h"
#include "spsolvert.h"
#include "nspsolvert.h"
#include "globalt.h"
#include "globmatt.h"
#include "transprint.h"
#include <string.h>
#include "npsolvert.h"

/**
   function solves homogeniztion for transport problems on PUC

   TKr, 17/08/2010
*/
void transport_homogenization ()
{
  long i,j;
  long ntm,ncomp;
  double puc_area;
  double *rhs,*lhs;
  matrix ltm,lm,kpuc,km,kM,kinv,khelp,cM;

  //  initiation of transport material models
  Tm->initmaterialmodels();
  //  nodes - integration points interpolation
  approximation_puc ();


  //---------------------------------------//
  //solving of fluctuation patr
  switch (Tp->tprob){
  case stationary_problem:{
    //stationary problem solution
    solve_stationary_problem ();
    break;
  }
  case nonlinear_stationary_problem:{
    solve_nonlinear_stationary_problem_pokus ();
    //solve_nonlinear_stationary_problem ();
    break;
  }
  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
  }
  }

  //---------------------------------------//
  //updating of unknown from master unknown, gradients and fluctuation part
  //  nodes - integration points interpolation
  approximation_puc ();

  //---------------------------------------//
  //homogenization
  //homogenization of conductivity matrix
  //computing of l matrix and l_t matrix
  ntm = Tp->ntm;
  ncomp = Tt->nodes[0].ncompgrad;
  allocm (Ndoft,ntm*ncomp,ltm);
  allocm (ntm*ncomp,Ndoft,lm);
  allocm (Ndoft,Ndoft,kpuc);
  allocm (Ndoft,Ndoft,kinv);
  allocm (Ndoft,ntm*ncomp,khelp);

  assemble_l_matrix (lm,ltm);
  
  /*
  if (Mesprt != 0){
    fprintf(Outt,"\n\n matice Lm\n");
    for (i=0;i<ntm*ncomp;i++){
      for (j=0;j<Ndoft;j++)
	fprintf(Outt,"%e  \n",lm[i][j]);
      fprintf(Outt,"\n");
    }

    fprintf(Outt,"\n\n matice Ltm\n");
    for (i=0;i<Ndoft;i++){
      for (j=0;j<ntm*ncomp;j++)
	fprintf(Outt,"%e  \n",ltm[i][j]);
      fprintf(Outt,"\n");
    }
  }
  */

  //conductivity matrix
  //assemble_conductivity_matrix (kpuc);//conductivity matrix of PUC stored as an array
  //conductivity matrix of PUC
  conductivity_matrix (0);

  //computing of average matrix (D_m in TRFEL notation)
  allocm (ntm*ncomp,ntm*ncomp,km);

  assemble_average_d_matrix(km,puc_area);
  
  //computing of macroscopic matrix (D_M in TRFEL notation)
  allocm (ntm*ncomp,ntm*ncomp,kM);  

  //contribution from fluctuation part
  //inversion of matrix K
  // lhs
  lhs = new double [kinv.m];
  nullv (lhs,kinv.m);
  // rhs
  rhs = new double [kinv.m];
  nullv (rhs,kinv.m);
  
  for(i=0;i<kinv.m;i++){
    nullv (rhs,kinv.m);
    nullv (lhs,kinv.m);
    rhs[i] = 1.0;
    Tp->ssle->solve_system (Gtt,Kmat,lhs,rhs,Outt);
    for(j=0;j<kinv.n;j++){
      kinv[j][i]=lhs[j];
    }
  }
  //inversion of matrix kpuc stored as an array
  //invm(kpuc,kinv,Tp->zero);

  /* //kontrola:
     matrix aux(kinv.m, kinv.n);
     mxm(kpuc, kinv, aux);
     printm(aux, Outt, 3, 11);
     destrm(aux);
  */

  mxm(kinv,ltm,khelp);
  mxm(lm,khelp,kM);
  cmulm(-1.0,kM);
  puc_area = 1.0/puc_area;
  cmulm(puc_area,kM);

  /*
  if (Mesprt != 0){
    fprintf(Outt,"\n\n matice D^m\n");
    for (i=0;i<ntm*ncomp;i++){
      for (j=0;j<ntm*ncomp;j++)
	fprintf(Outt,"%e  \n",km[i][j]);
      fprintf(Outt,"\n");
    }
    
    fprintf(Outt,"\n\n fluktuacni prispevky D\n");
    for (i=0;i<ntm*ncomp;i++){
      for (j=0;j<ntm*ncomp;j++)
	fprintf(Outt,"%e  \n",kM[i][j]);
      fprintf(Outt,"\n");
    }
  }
  */
  
  //final macroscopic conductivity matrix (D_M in TRFEL notation)
  addm(km,kM,kM);

  //computing of macroscopic cpacity matrix (C_M in TRFEL notation)
  allocm (ntm,ntm,cM);
  //homogenization of capacity matrix
  //computing of average matrix
  assemble_average_c_matrix(cM);
  
  
  if (Mesprt != 0){
    fprintf(Outt,"\n\n vysledna matice D^M\n");
    for (i=0;i<ntm*ncomp;i++){
      for (j=0;j<ntm*ncomp;j++)
	fprintf(Outt,"%e  \n",kM[i][j]);
      fprintf(Outt,"\n");
    }
    
    fprintf(Outt,"\n\n vysledna matice C^M\n");
    for (i=0;i<ntm;i++){
      for (j=0;j<ntm;j++)
	fprintf(Outt,"%e  \n",cM[i][j]);
      fprintf(Outt,"\n");
    }
  }
  

  destrm (lm);
  destrm (ltm);
  destrm (kpuc);
  destrm (kinv);
  destrm (km);
  destrm (kM);
  destrm (cM);
  destrm (khelp);
  delete [] rhs;
  delete [] lhs;
}

/**
   function solves homogenization for transport problems on PUC and returns overall (homogenized) conductivity and capacity matrices 

   @param unkn_r - unknows and gradients of unknowns in integration point (in master node on PUC) stored as an array (one dimensional)
   @param matrix_k - homogenized conductivity matrix of PUC stored as an array (one dimensional)
   @param matrix_c - homogenized capacity matrix of PUC stored as an array (one dimensional)

   TKr, 17/08/2010
*/
void paral_transport_homogenization (double *unkn_r,double *matrix_k,double *matrix_c)
{
  long i,j,k;
  long ntm,ncomp;
  double puc_area;
  double *rhs,*lhs;
  matrix ltm,lm,kpuc,km,kM,kinv,khelp,cM;

  /*
  fprintf (stdout,"\n paral_transport_homogenization");
  for (long ijk=0;ijk<6;ijk++){
    fprintf (stdout,"\n unkn_r %ld   %le",ijk,unkn_r[ijk]);
  }
  */
  
  //  initiation of transport material models
  Tm->initmaterialmodels();

  //initiation of master node unknowns and gradients
  ntm = Tp->ntm;
  ncomp = Tt->nodes[0].ncompgrad;
  k = -1;
  for (i=0;i<ntm;i++){
    k++;
    Tb->lc[i].masterval = unkn_r[k];
    for (j=0;j<ncomp;j++){
      k++;
      Tb->lc[i].mastergrad[j] = unkn_r[k];
    }
  }
  
  /*
  fprintf (stdout,"\n Tb->lc[0].masterval   %le",Tb->lc[0].masterval);
  fprintf (stdout,"\n Tb->lc[1].masterval   %le",Tb->lc[1].masterval);
  fprintf (stdout,"\n Tb->lc[0].mastergrad[0]  %le",Tb->lc[0].mastergrad[0]);
  fprintf (stdout,"\n Tb->lc[0].mastergrad[1]  %le",Tb->lc[0].mastergrad[1]);
  fprintf (stdout,"\n Tb->lc[1].mastergrad[0]  %le",Tb->lc[1].mastergrad[0]);
  fprintf (stdout,"\n Tb->lc[1].mastergrad[1]  %le",Tb->lc[1].mastergrad[1]);
  */
  
  //  nodes - integration points interpolation
  approximation_puc ();

  //---------------------------------------//
  //solving of fluctuation patr
  switch (Tp->tprob){
  case stationary_problem:{
    //stationary problem solution
    solve_stationary_problem ();
    break;
  }
  case nonlinear_stationary_problem:{
    solve_nonlinear_stationary_problem_pokus ();
    //solve_nonlinear_stationary_problem ();
    break;
  }
  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
  }
  }

  //---------------------------------------//
  //updating of unknown from master unknown, gradients and fluctuation part
  //  nodes - integration points interpolation
  approximation_puc ();

  //---------------------------------------//
  //homogenization
  //homogenization of conductivity matrix
  //computing of l matrix and l_t matrix
  allocm (Ndoft,ntm*ncomp,ltm);
  allocm (ntm*ncomp,Ndoft,lm);
  allocm (Ndoft,Ndoft,kpuc);
  allocm (Ndoft,Ndoft,kinv);
  allocm (Ndoft,ntm*ncomp,khelp);

  assemble_l_matrix (lm,ltm);

  //conductivity matrix
  //conductivity matrix of PUC
  conductivity_matrix (0);

  //computing of average matrix (D_m in TRFEL notation)
  allocm (ntm*ncomp,ntm*ncomp,km);

  assemble_average_d_matrix(km,puc_area);
  
  //computing of macroscopic matrix (D_M in TRFEL notation)
  allocm (ntm*ncomp,ntm*ncomp,kM);  

  //contribution from fluctuation part
  //inversion of matrix K
  // lhs
  lhs = new double [kinv.m];
  nullv (lhs,kinv.m);
  // rhs
  rhs = new double [kinv.m];
  nullv (rhs,kinv.m);
  
  for(i=0;i<kinv.m;i++){
    nullv (rhs,kinv.m);
    nullv (lhs,kinv.m);
    rhs[i] = 1.0;
    Tp->ssle->solve_system (Gtt,Kmat,lhs,rhs,Outt);
    for(j=0;j<kinv.n;j++){
      kinv[j][i]=lhs[j];
    }
  }

  mxm(kinv,ltm,khelp);
  mxm(lm,khelp,kM);
  cmulm(-1.0,kM);
  puc_area = 1.0/puc_area;
  cmulm(puc_area,kM);

  //final macroscopic conductivity matrix (D_M in TRFEL notation)
  addm(km,kM,kM);

  //computing of macroscopic cpacity matrix (C_M in TRFEL notation)
  allocm (ntm,ntm,cM);
  //homogenization of capacity matrix
  //computing of average matrix
  assemble_average_c_matrix(cM);

  //storing of overall properties
  //conductivity matrix
  k = 0;
  for (i=0;i<ntm*ncomp;i++){
    for (j=0;j<ntm*ncomp;j++){
      matrix_k[k] = kM[i][j];
      k++;
    }
  }

  //capacity matrix
  k = 0;
  for (i=0;i<ntm;i++){
    for (j=0;j<ntm;j++){
      matrix_c[k] = cM[i][j];
      k++;
    }
  }

  destrm (lm);
  destrm (ltm);
  destrm (kpuc);
  destrm (kinv);
  destrm (km);
  destrm (kM);
  destrm (cM);
  destrm (khelp);
  delete [] rhs;
  delete [] lhs;

}
