#ifndef RICHARDS_H
#define RICHARDS_H

#include "genfile.h"

class richards
{
 public:
  richards();    //constructor
  ~richards();   //destructor
  
  void read(XFILE *in);
  void print(FILE *out);
  
  
  ///  function computes hydraulic conductivity
  double kk_value (double h);
  ///  function computes the derivative of K with respect to h (dK/dh van Genuchten).
  double dkkdh_value (double h);
  ///  function computes second derivative of K with respect to h (d^2K/dh^2 van Genuchten).
  double ddkkddh_value (double h);
  ///  function computes capacity (van Genuchten).
  double c_value (double h);
  ///  function computes derivative of capacity with respect to hydraulic head (van Genuchten).
  double dcdh_value (double h);
  
  void matcond (matrix &d,long ri,long ci,long ipp);
  void matcap (double &c,long ri,long ci,long ipp);
  
  void matcond1d (matrix &d,long ri,long ci,long ipp);
  void matcond2d (matrix &d,long ri,long ci,long ipp);
  void matcond3d (matrix &d,long ri,long ci,long ipp);

  void matcond2 (matrix &d,long ri,long ci,long ipp);
  void matcond1d_2 (matrix &d,long ri,long ci,long ipp);
  void matcond2d_2 (matrix &d,long ri,long ci,long ipp);
  void matcond3d_2 (matrix &d,long ri,long ci,long ipp);
  

  ///  parameters of van Genuchten model
  double alpha;
  double n;
  double m;
   
  ///  saturated hydraulic conductivity
  double kks;
  ///  saturated water content
  double thetas;
  ///  residual water content
  double thetar;
  ///  specific storage
  double storage;

};

#endif
