#ifndef TRANSMAT_H
#define TRANSMAT_H

#include <stdio.h>
#include "iotools.h"
#include "aliast.h"
#include "selection.h"
#include "intpointst.h"
#include "isotrmat.h"
#include "sejtkrmat.h"
#include "nlisotrmat.h"
#include "cerny_concrete.h"
#include "bazped.h"
#include "baroghelB.h"
#include "concreteB.h"
#include "pedersen.h"
#include "carbmat1.h"
#include "carbmat1.h"
#include "kunzel.h"
#include "kunzel2.h"
#include "glasgowmat.h"
#include "genfile.h"
#include "C60baroghel.h"
#include "C30baroghel.h"
#include "o30bazant.h"
#include "C60bazant.h"
#include "C30bazant.h"
#include "saltmat1.h"
#include "saltmat2.h"
#include "saltmat3.h"
#include "saltmat4.h"
#include "grunewaldmat.h"
#include "aepointst.h"
#include "soil1mat.h"
#include "discisotrmat.h"
#include "cemhydmat.h"
#include "devriesmat.h"
#include "lincoupmat.h"
#include "discmat.h"
#include "consol_awf1.h"
#include "consol_awf2.h"
#include "millymat.h"
#include "homogmat.h"
#include "radiationmat.h"
#include "richards.h"

class transmat
{
 public:
  transmat (void);
  ~transmat (void);


  // *********************************************
  // Functions for alocation and reading materials
  // *********************************************

  long intpnum (void);


  void readip (FILE *in);


  void intpointalloc ();


  void alloc_nontransq(long n);


  void intpointinit ();

    
  void assemble_dof_nameord(namevart *dofname, long ntm, long *var_dofid, long tnkv);

  void give_dof_names(long ipp, namevart *dofname, long ntm);


  void read (XFILE *in);


  void print (FILE *out);


  void readmatchar (XFILE *in);


  void printmatchar (FILE *out);


  void initmaterialmodels (void);


  void initvalues (long ipp,long im,long ido);


  void updateipval (void);


  void updateipvalmat (long ipp,long im,long ido);

  // ***********************************************************
  // Functions for conductivity and capacity matrices and fluxes
  // ***********************************************************

  void matcond (matrix &d,long ipp,long ri,long ci);


  void matcond2 (matrix &d,long ipp,long ri,long ci);


  double capcoeff (long ipp,long ri,long ci);


  void volume_rhs (matrix &d,long ipp,long ri,long ci,long ncomp);


  void computenlfluxes (long lcid,long ipp);


  void flux_contributions (long ipp);


  void transmission_transcoeff(double &new_trc,double trc,long ri,long ci,long nn,long bc,long ipp);


  void transmission_transcoeff(double &new_trc,double trc,long ri,long ci,long nn,long bc,long ipp,int flag);


  void transmission_nodval(double &new_nodval,double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp);


  void transmission_flux(double &flux,double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp);


  void values_correction (vector &nv,long ipp);
  
  
  void values_transformation (mattypet mti,long idmi,double *inv,
			      double *iinv,mattypet mto,long idmo,double *outv,double *ioutv,double *jum);
  
  void fluxcomparing (long ipp);


  void aux_values (long lcid);
  
  void mat_aux_values (long ipp);

  
  double give_extinction_coeff (mattypet mt,long matid);


  // ************************************************
  // Functions for gradients and fluxes, other values
  // ************************************************

  void storegrad (long lcid,long ipp,vector &gr);


  void givegrad (long lcid,long ipp,vector &gr);


  void storeflux (long lcid,long ipp,vector &fl);


  void givefluxes (long lcid,long ipp,vector &fl);

  
  void storeother (long ipp,long ncomp,double *comp);


  void storeeqother (long ipp,long ncompeq,double *compeq);


  void giveother (long ipp,long ncomp,double *comp);


  void giveeqother (long ipp,long ncompeq,double *compeq);


  long givencompother ();


  long givencompeqother (long ipp,long im);


  double givecompother(long compother,long ipp,double *r);


  void give_othervalue_name(FILE *out,long ipp,long compother);


  void give_eqothervalue_name(FILE *out,long ipp,long compeqother);
  
  ///  function returns the temperature in integration point
  double give_temperature (long ipp);

  ///  function returns the initial temperature in integration point
  double give_inittemperature (long ipp);

  ///  function returns the relative humidity in integration point
  double give_rel_hum (long ipp);

  /// returns required transport quantity for passed to MEFEL
  double givetransq(nonmechquant qt, long ipp);

  /// returns water pressure
  double give_water_pressure(long ipp);

  /// returns pore pressure
  double give_pore_pressure(long ipp);
  
  /// returns volumetric moisture content
    double give_vol_moist (long ipp);
  
  // **********************************************************************
  // Functions for retrieving of non-transport quantities from int. points
  // **********************************************************************

  /// searches for the number and types of required non-transport quantities at all material models
  long search_reqntq(nontransquant* &rntq);

  /// marks required types of non-transport quantities by the given material model in the given int. point
  void give_reqntq(long ipp, long *antq);

  /// returns given non-transport quantity from nontransq array on given integration point
  double givenontransq (nontransquant qt, long ipp);

  /// stores given non-transport quantity from nontransq array on given integration point
  void storenontransq (nontransquant qt, long ipp, double val);

  /// returns status of non-transport quantity (stored in nontransq 1, otherwise 0)
  long givestatusntq (nontransquant qt);

  /// returns status of non-transport quantity (stored in nontransq 1, otherwise 0)
  long givenontransqid (nontransquant qt);


  
  // ********************
  // Functions for backup
  // ********************

  long cycle_detection (double *r,double *pr,double *ppr,long ipp);


  void save_intpointst_txt (FILE *aux, sel &selelems, sel *selother);


  void save_intpointst_txt (long ni, sel &selelems, sel *selother);


  void save_intpointst_bin (FILE *aux, sel &selelems, sel *selother);


  void save_intpointst_bin (long ni, sel &selelems, sel *selother);


  void restore_intpointst_txt (FILE *aux, sel &selelems, sel *selother, long **selid);


  void restore_intpointst_txt (sel &selelems, sel *selother, long **selid);


  void restore_intpointst_bin (FILE *aux, sel &selelems, sel *selother, long **selid);


  void restore_intpointst_bin (sel &selelems, sel *selother, long **selid);
  
  
  
  ///  number of material types
  long nmt;
  ///  material types
  mattypet *mattype;
  ///  numbers of particular material types
  long *numtype;
  ///  total number of all integration points
  long tnip;
  ///  pointer to integration points
  intpointst *ip;
  ///  integration point - element correspondation
  ///  elip[number of int point] = number of element
  long *elip;
  
  // ******************
  //  material models
  // ******************

  // one-phase medium - heat transfer
  //  isotropic material
  isotrmat *itrm;
  //  one-phase medium - heat transfer
  //  nonlinear isotropic material
  nlisotrmat *nlitrm;
  //  artificial material obtained from homogenization
  homogmat *hommat;

  //  saturated-unsaturated one-phase flow in deforming medium
  sejtkrmat *sejtkrm;
  con_awf1mat *consol_awf1;

  ///  one-phase medium - heat transfer
  ///  isotropic material with jumps
  discisotrmat *ditrm;

  carbmat1 *carb1;

  // concrete from Cerny
  cernymat *cernym;
  
  // heat-moisture transport in concrete
  //two-phase medium
  kunmat *kun;
  kunmat2 *kun2;
  grunewaldmat *grunw;
  bazpedmat *bazped;
  pedmat *ped;
  devriesmat *dvries;
  discmat *sdmat;
  con_awf2mat *consol_awf2;
  millymat *mill;
  
  //three-phase medium
  baroghelmat *baroghel;
  concreteBmat *concrete;
  C60barmat  *C60baroghel;
  C30barmat  *C30baroghel;
  o30bazmat  *o30bazant;
  C60bazmat  *C60bazant;
  C30bazmat  *C30bazant;
  glasgowmat *tench;
  
  // moisture-salt transport in porous mat.
  //two-phase medium
  saltmat1 *salt1;
  
  //  three-phase
  saltmat2 *salt2;
  saltmat3 *salt3;
  //  four-phase materials
  saltmat4 *salt4;
  
  radiationmat *radmat;
  richards *richar;
  
  soil1mat *soil1;

  //cemhyd model
  cemhydmat *cemhydr;
  
  //  linear coupled model (for arbitrary number of matters)
  lincoupmat *lcmat;
  
  
  ///  strain/stress points
  aepointst grad,flux;
  
  
  ///  array containing initial values of selected unknown at integration points
  ///  initval[number of integ. point] - initial value at the required integration point
  double *initval;
  ///  array containing nontransport quantities used for coupled problems
  ///  nontransq[quantity id][integ. point number] - nontransport quantity at the required integration point
  double **nontransq;
  ///  number of nontransport quantities
  long nntq;
  /// total number of known(=implemented) non-transport quantities
  static const long tnkntq = sizeof(nontransquantstr)/sizeof(*nontransquantstr);
  /// array of indices in nontransq for particular non-transport quantities
  long ntqid[tnkntq];
  /// array with ordering of used non-transport quantities
  nontransquant *ntqo;
};

#endif
